#include <stdio.h>

void print_flag(){
    char flag[0x100];
    FILE *fp;
    fp = fopen("flag.txt","r");
    fgets(flag, sizeof(flag), fp);
    puts(flag);
}

int main(){
    setvbuf(stdin, NULL, 2, 0);
    setvbuf(stdout, NULL, 2, 0);
    puts(
"=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=\n"
"So if you remember babybof, there was a line,\n"
"`gcc -no-pie -fno-stack-protector babybof.c -o babybof`\n"
"What does this line do? We can break it down\n"
"\n"
" - gcc: GNU project C and C++ compiler. This command allows us to turn our c\n"
"code into a functioning program\n"
" - -no-pie: An option inside gcc to not produce a position independent\n"
"executable (PIE).\n"
" - -fno-stack-protector: Another option inside gcc. Documentation states \"-fno\""
"represents the opposite of the options with \"-f\" or \"-W\". In this case,\n"
"it is the negation of -fstack-protector. Documentation states it \"emits\n"
"extra code to check for buffer overflows by adding a guard variable to\n"
"functions\"."
" - babybof.c: The c file to compile.\n"
" - -o: Place out of file.\n"
" - babybof: The file to output the compiled code in.\n"
"\n"
"So what the command does is take `babybof.c`, compiles it through `gcc`\n"
"with the flags `-no-pie` and `-fno-stack-protector` and ouputs (-o) it to\n"
"`babybof`.\n"
"Press enter to continue..."
);
getc(stdin);
puts(
"=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=\n"
"So what does `-no-pie` and `-fno-stack-protector` really do? In high level\n"
"terms, these disable two security features in modern programs, PIE, and\n"
"Stack Canary.\n"
"\n"
"PIE: Position independent Executable. Determines how the binary will be loaded.\n"
"Specifically, if PIE is enabled, it will load the program at a random base\n"
"address. If PIE is disabled, it will load the program at a fixed base address.\n"
"For example, in `babyret2win`, PIE is disabled and the address of print_flag is\n"
"0x4011d6. If PIE is enabled, print_flag will not always be at 0x4011d6. Thus,\n"
"if PIE was enabled, we would not be able to solve `babyret2win` as we would not\n"
"know where the address of print_flag be. One thing to note is the offset\n"
"between things in a program will stay constant. So, the offset between `main`\n"
"and `print_flag` remains constant and if you know the address of `main`,\n"
"you know the address of `print_flag` and vice versa.\n"
"\n"
"Stack Canary: Adds an extra variable in every function called a cookie. This\n"
"variable is checked before the function returns. Usually, it looks like\n"
"\n"
"|        |\n"
"|--------|\n"
"|????????|  <- name\n"
"|????????|\n"
"|????????|\n"
"|????????|\n"
"|--------|\n"
"|0000????|  <- is_admin (4 bytes long + endianness)\n"
"|--------|\n"
"|????????|  <- Stack Cookie (Note there's no need for padding)\n"
"|--------|\n"
"|????????|  <- Base Pointer\n"
"|--------|\n"
"|????????|  <- Return Pointer\n"
"|========|\n"
"|   ...  |\n"
"+--------+\n"
"\n"
"And at the end of the program it checks if the stack cookies equals a random\n"
"value. If not, the program will immidiately exit. Note if you try to perform\n"
"a buffer overflow like in babyret2win, the program will see the stack cookie\n"
"is modified, and will exit. The only bypass to this is to leak the stack\n"
"cookie.\n"
"Press enter to continue..."
);
getc(stdin);
puts(
"=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=\n"
"Now, let's consider if we compile this with `gcc babycookie.c -o babycookie`\n"
"In order to do what we did in babyret2win, we now need to leak the stack\n"
"cookie and some program address to defeat PIE. Luckly, this gets to our next\n"
"vulnerability, format strings. `printf` is a function provided by c which\n"
"allows you to easily format your output. `printf` is also a very powerful\n"
"functions and is actually turning complete. `printf` has different format\n"
"strings you can use, usually in the form of \"%?\". For example, %d would\n"
"print an integer, %x would print an integer as hexadecimal, and %s would\n"
"print a string.\n"
"\n"
"Now, documentation states the function signiture of printf is\n"
"int printf(const char *format, ...). The ... means it can take in as\n"
"many additional arguements as needed. So, I can put something like\n"
"printf(\"%d %d\n\",1,2,3,4,5,6,7,8,9,10,11,12);\n"
"and it would print out `1 2`\n"
"Now, where how do you pass these extra arguements in to a function?\n"
"Well, the way variables are passed as arguements to other functions\n"
"Are first, through registers, then through the stack. For example,\n"
"compiling the printf statement and looking at the disassembly, we see\n"
"\n"
"0000000000401136 <main>:\n"
"  401136:       f3 0f 1e fa             endbr64\n"
"  40113a:       55                      push   rbp\n"
"  40113b:       48 89 e5                mov    rbp,rsp\n"
"  40113e:       48 83 ec 08             sub    rsp,0x8\n"
"  401142:       6a 0c                   push   0xc\n"
"  401144:       6a 0b                   push   0xb\n"
"  401146:       6a 0a                   push   0xa\n"
"  401148:       6a 09                   push   0x9\n"
"  40114a:       6a 08                   push   0x8\n"
"  40114c:       6a 07                   push   0x7\n"
"  40114e:       6a 06                   push   0x6\n"
"  401150:       41 b9 05 00 00 00       mov    r9d,0x5\n"
"  401156:       41 b8 04 00 00 00       mov    r8d,0x4\n"
"  40115c:       b9 03 00 00 00          mov    ecx,0x3\n"
"  401161:       ba 02 00 00 00          mov    edx,0x2\n"
"  401166:       be 01 00 00 00          mov    esi,0x1\n"
"  40116b:       48 8d 05 92 0e 00 00    lea    rax,[rip+0xe92]        # 402004 <_IO_stdin_used+0x4>\n"
"  401172:       48 89 c7                mov    rdi,rax\n"
"  401175:       b8 00 00 00 00          mov    eax,0x0\n"
"  40117a:       e8 c1 fe ff ff          call   401040 <printf@plt>\n"
"  40117f:       48 83 c4 40             add    rsp,0x40\n"
"  401183:       b8 00 00 00 00          mov    eax,0x0\n"
"  401188:       c9                      leave\n"
"  401189:       c3                      ret\n"
"So, it moves 1-5 into registers and pushes the rest of the arguements onto\n"
"the stack. Now, if we have something like `printf(name)`, we control the\n"
"format strings and whatever are in the registers and on the stack are our\n"
"arguements. This include several different things such as strings, addresses,\n"
"and pointers. What this means is that we can use the format string to leak\n"
"values from the stack. And one thing we know exists on the stack is the\n"
"cookie and an address to our binary.\n"
"Note: One useful feature of printf is the fact you can print values at\n"
"absolute offsets. printf(\"%6$d\",1,2,3,4,5,6,7,8,9,10) would print `6`\n"
"Press enter to continue..."
);
getc(stdin);
puts(
"=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=\n"
"So, you have a buffer overflow with gets, and a leak with printf.\n"
"With the leak you are able to leak where the address of the binary is\n"
"and the value of the stack cookie. From there, you can use the buffer\n"
"overflow to leak the stack cookie and jump to print_flag.\n"
"Are you ready?\n"
"Press enter to continue..."
);
getc(stdin);

    int is_admin=0;
    char data[0x20];
    char name[0x20];
    puts("What is your name?");
    fgets(name, 0x20, stdin);
    printf("Hi ");
    printf(name);
    puts("");
    puts("What do you want to tell me?");
    gets(data);
}
